@@ -1,9 +1,9 @@
-// +build circuitplay_express
+//go:build circuitplay_express
 
 package machine
 
 // used to reset into bootloader
-const RESET_MAGIC_VALUE = 0xf01669ef
+const resetMagicValue = 0xf01669ef
 
 // GPIO Pins
 const (
@@ -67,6 +67,13 @@ const (
 	UART_RX_PIN = PB09 // PORTB
 )
 
+// UART1 on the Circuit Playground Express.
+var (
+	UART1 = &sercomUSART4
+
+	DefaultUART = UART1
+)
+
 // I2C pins
 const (
 	SDA_PIN = PB02 // I2C0 external
@@ -76,6 +83,12 @@ const (
 	SCL1_PIN = PA01 // I2C1 internal
 )
 
+// I2C on the Circuit Playground Express.
+var (
+	I2C0 = sercomI2CM5 // external device
+	I2C1 = sercomI2CM1 // internal device
+)
+
 // SPI pins (internal flash)
 const (
 	SPI0_SCK_PIN = PA21 // SCK: SERCOM3/PAD[3]
@@ -83,6 +96,9 @@ const (
 	SPI0_SDI_PIN = PA16 // SDI: SERCOM3/PAD[0]
 )
 
+// SPI on the Circuit Playground Express.
+var SPI0 = sercomSPIM3
+
 // I2S pins
 const (
 	I2S_SCK_PIN = PA10
