@@ -87,8 +87,17 @@ const (
 	NINA_GPIO0  Pin = GPIO2
 	NINA_RESETN Pin = GPIO3
 
-	NINA_TX Pin = GPIO9
-	NINA_RX Pin = GPIO8
+	NINA_TX  Pin = GPIO8
+	NINA_RX  Pin = GPIO9
+	NINA_CTS Pin = GPIO10
+	NINA_RTS Pin = GPIO11
+)
+
+// NINA-W102 settings
+const (
+	NINA_BAUDRATE         = 115200
+	NINA_RESET_INVERTED   = true
+	NINA_SOFT_FLOWCONTROL = false
 )
 
 // Onboard crystal oscillator frequency, in MHz.
@@ -123,10 +132,20 @@ var (
 		Buffer: NewRingBuffer(),
 		Bus:    rp.UART0,
 	}
+
+	UART1  = &_UART1
+	_UART1 = UART{
+		Buffer: NewRingBuffer(),
+		Bus:    rp.UART1,
+	}
+
+	// UART_NINA on the Arduino Nano RP2040 connects to the NINA HCI.
+	UART_NINA = UART1
 )
 
 var DefaultUART = UART0
 
 func init() {
 	UART0.Interrupt = interrupt.New(rp.IRQ_UART0_IRQ, _UART0.handleInterrupt)
+	UART1.Interrupt = interrupt.New(rp.IRQ_UART1_IRQ, _UART1.handleInterrupt)
 }
