@@ -29,6 +29,7 @@
 //     ptrTo        *typeStruct
 //     elem         *typeStruct // element type of the array
 //     arrayLen     uintptr     // length of the array (this is part of the type)
+//     slicePtr     *typeStruct // pointer to []T type
 // - map types (this is still missing the key and element types)
 //     meta         uint8
 //     nmethods     uint16 (0)
@@ -427,6 +428,7 @@ type arrayType struct {
 	ptrTo     *rawType
 	elem      *rawType
 	arrayLen  uintptr
+	slicePtr  *rawType
 }
 
 type mapType struct {
@@ -1292,6 +1294,10 @@ func MapOf(key, value Type) Type {
 	panic("unimplemented: reflect.MapOf()")
 }
 
+func FuncOf(in, out []Type, variadic bool) Type {
+	panic("unimplemented: reflect.FuncOf()")
+}
+
 const maxVarintLen32 = 5
 
 // encoding/binary.Uvarint, specialized for uint32
@@ -1307,3 +1313,9 @@ func uvarint32(buf []byte) (uint32, int) {
 	}
 	return 0, 0
 }
+
+// TypeFor returns the [Type] that represents the type argument T.
+func TypeFor[T any]() Type {
+	// This function was copied from the Go 1.22 source tree.
+	return TypeOf((*T)(nil)).Elem()
+}
