@@ -124,10 +124,9 @@ func (v Value) IsZero() bool {
 	case Uint, Uint8, Uint16, Uint32, Uint64, Uintptr:
 		return v.Uint() == 0
 	case Float32, Float64:
-		return math.Float64bits(v.Float()) == 0
+		return v.Float() == 0
 	case Complex64, Complex128:
-		c := v.Complex()
-		return math.Float64bits(real(c)) == 0 && math.Float64bits(imag(c)) == 0
+		return v.Complex() == 0
 	case Array:
 		for i := 0; i < v.Len(); i++ {
 			if !v.Index(i).IsZero() {
@@ -141,7 +140,7 @@ func (v Value) IsZero() bool {
 		return v.Len() == 0
 	case Struct:
 		for i := 0; i < v.NumField(); i++ {
-			if !v.Field(i).IsZero() {
+			if !v.Field(i).IsZero() && v.Type().Field(i).Name != "_" {
 				return false
 			}
 		}
@@ -555,8 +554,26 @@ func (v Value) Slice(i, j int) Value {
 		}
 
 	case Array:
-		// TODO(dgryski): can't do this yet because the resulting value needs type slice of v.elem(), not array of v.elem().
-		// need to be able to look up this "new" type so pointer equality of types still works
+		v.checkAddressable()
+		buf, length := buflen(v)
+		i, j := uintptr(i), uintptr(j)
+		if j < i || length < j {
+			slicePanic()
+		}
+
+		elemSize := v.typecode.underlying().elem().Size()
+
+		var hdr sliceHeader
+		hdr.len = j - i
+		hdr.cap = length - i
+		hdr.data = unsafe.Add(buf, i*elemSize)
+
+		sliceType := (*arrayType)(unsafe.Pointer(v.typecode.underlying())).slicePtr
+		return Value{
+			typecode: sliceType,
+			value:    unsafe.Pointer(&hdr),
+			flags:    v.flags,
+		}
 
 	case String:
 		i, j := uintptr(i), uintptr(j)
@@ -604,9 +621,26 @@ func (v Value) Slice3(i, j, k int) Value {
 		}
 
 	case Array:
-		// TODO(dgryski): can't do this yet because the resulting value needs type v.elem(), not array of v.elem().
-		// need to be able to look up this "new" type so pointer equality of types still works
+		v.checkAddressable()
+		buf, length := buflen(v)
+		i, j, k := uintptr(i), uintptr(j), uintptr(k)
+		if j < i || k < j || length < k {
+			slicePanic()
+		}
 
+		elemSize := v.typecode.underlying().elem().Size()
+
+		var hdr sliceHeader
+		hdr.len = j - i
+		hdr.cap = k - i
+		hdr.data = unsafe.Add(buf, i*elemSize)
+
+		sliceType := (*arrayType)(unsafe.Pointer(v.typecode.underlying())).slicePtr
+		return Value{
+			typecode: sliceType,
+			value:    unsafe.Pointer(&hdr),
+			flags:    v.flags,
+		}
 	}
 
 	panic("unimplemented: (reflect.Value).Slice3()")
@@ -1052,6 +1086,13 @@ func (v Value) Set(x Value) {
 	}
 
 	if v.typecode.Kind() == Interface && x.typecode.Kind() != Interface {
+		// move the value of x back into the interface, if possible
+		if x.isIndirect() && x.typecode.Size() <= unsafe.Sizeof(uintptr(0)) {
+			var value uintptr
+			memcpy(unsafe.Pointer(&value), x.value, x.typecode.Size())
+			x.value = unsafe.Pointer(value)
+		}
+
 		intf := composeInterface(unsafe.Pointer(x.typecode), x.value)
 		x = Value{
 			typecode: v.typecode,
@@ -1970,6 +2011,10 @@ func (v Value) Call(in []Value) []Value {
 	panic("unimplemented: (reflect.Value).Call()")
 }
 
+func (v Value) CallSlice(in []Value) []Value {
+	panic("unimplemented: (reflect.Value).CallSlice()")
+}
+
 func (v Value) Method(i int) Value {
 	panic("unimplemented: (reflect.Value).Method()")
 }
