@@ -1,5 +1,4 @@
-//go:build nrf
-// +build nrf
+//go:build nrf && !nrf52840
 
 package runtime
 
@@ -28,7 +27,7 @@ func main() {
 }
 
 func init() {
-	machine.Serial.Configure(machine.UARTConfig{})
+	machine.InitSerial()
 	initLFCLK()
 	initRTC()
 }
