@@ -5,6 +5,10 @@ type Frames struct {
 }
 
 type Frame struct {
+	PC uintptr
+
+	Func *Func
+
 	Function string
 
 	File string
