@@ -68,6 +68,7 @@ const (
 	EISDIR      Errno = 21
 	EINVAL      Errno = 22
 	EMFILE      Errno = 24
+	EROFS       Errno = 30
 	EPIPE       Errno = 32
 	EAGAIN      Errno = 35
 	ENOTCONN    Errno = 57
@@ -147,6 +148,11 @@ type Timespec struct {
 	Nsec int64
 }
 
+// Unix returns the time stored in ts as seconds plus nanoseconds.
+func (ts *Timespec) Unix() (sec int64, nsec int64) {
+	return int64(ts.Sec), int64(ts.Nsec)
+}
+
 // Source: upstream ztypes_darwin_amd64.go
 type Dirent struct {
 	Ino       uint64
@@ -158,7 +164,6 @@ type Dirent struct {
 	Pad_cgo_0 [3]byte
 }
 
-// Go chose Linux's field names for Stat_t, see https://github.com/golang/go/issues/31735
 type Stat_t struct {
 	Dev       int32
 	Mode      uint16
@@ -168,10 +173,10 @@ type Stat_t struct {
 	Gid       uint32
 	Rdev      int32
 	Pad_cgo_0 [4]byte
-	Atim      Timespec
-	Mtim      Timespec
-	Ctim      Timespec
-	Btim      Timespec
+	Atimespec Timespec
+	Mtimespec Timespec
+	Ctimespec Timespec
+	Btimespec Timespec
 	Size      int64
 	Blocks    int64
 	Blksize   int32
@@ -263,6 +268,23 @@ func Pipe2(fds []int, flags int) (err error) {
 	return
 }
 
+func Chmod(path string, mode uint32) (err error) {
+	data := cstring(path)
+	fail := int(libc_chmod(&data[0], mode))
+	if fail < 0 {
+		err = getErrno()
+	}
+	return
+}
+
+func closedir(dir uintptr) (err error) {
+	e := libc_closedir(unsafe.Pointer(dir))
+	if e != 0 {
+		err = getErrno()
+	}
+	return
+}
+
 func readdir_r(dir uintptr, entry *Dirent, result **Dirent) (err error) {
 	e1 := libc_readdir_r(unsafe.Pointer(dir), unsafe.Pointer(entry), unsafe.Pointer(result))
 	if e1 != 0 {
